#include "SevSeg.h"
SevSeg sevseg; //Instantiate a seven segment controller object

//#define DEBUG 1
#define DEBUG 0

#define coinPin 0 //interrupt 0, digital PIN 2

//array com os valores das moedas aceites
static float valuesCoins[] = {0.05, 0.10, 0.20};

volatile float totalAmount = 0;

// animation stuff
const long interval = 50;
unsigned long previousMillis = 0;
int currentPos = 0;


float valueToOpen = 0;

int lockCode = 941;

/* 
 * this is the value to display on the 7-segment display 
 * Because we can't display float values, but will control where
 * the dot will display - more info below
 */
int valueToDisplay = 0;

void setup() {
  Serial.begin(115200);
  //create an interrupt
  attachInterrupt (coinPin, impulseDetected, RISING);
  /*
   * randomize coins and value needed to open the cache
   */
  randomSeed(analogRead(0));
  int numCoins = random (2, 8); //7 coins is the max, always at least 2
  for (int i = 0; i < numCoins; i++) {
    float valueArray = valuesCoins[random(0,3)];
    valueToOpen = valueToOpen + valueArray;
  }

  //7-segment stuff
  byte numDigits = 4;
  byte digitPins[] = {A4, A3, A2, A1};
  byte segmentPins[] = {6, 7, 8, 9, 10, 11, 12, 13};
  bool resistorsOnSegments = false; // 'false' means resistors are on digit pins
  byte hardwareConfig = COMMON_ANODE; // See README.md for options
  bool updateWithDelays = false; // Default. Recommended
  bool leadingZeros = true; // Use 'true' if you'd like to keep the leading zeros
  
  sevseg.begin(hardwareConfig, numDigits, digitPins, segmentPins, resistorsOnSegments, updateWithDelays, leadingZeros);
  sevseg.setBrightness(90);
  delay(500);
  //startup animation
  animation2();
  
}

void loop() {
  //show the value that we need to insert
  valueToDisplay = valueToOpen * 100;
  sevseg.setNumber (valueToDisplay, 3);
  sevseg.refreshDisplay();
  /* 
  Serial.print("value display:" );
  Serial.println(valueToDisplay);
  Serial.print("value to open:" );
  Serial.println(valueToOpen);
  */
  if (valueToOpen <= 0) {
    //cache aberta
    sevseg.blank();
    delay (1000);
    animation1();
    while (1) {
      //stay here forever
      //mostrar o codigo do cadeado
      sevseg.setNumber (lockCode);
      sevseg.refreshDisplay();
    }
  }
}

void impulseDetected() {
  cli();
  //For every impulse, we remove 0.05 from the value to open
  valueToOpen = valueToOpen - 0.05;
  //totalAmount = totalAmount + 0.05;
  if (DEBUG) {
    Serial.print ("total: ");
    Serial.println (valueToOpen);
    Serial.print("Display: ");
    Serial.println (valueToOpen * 100);
    //Serial.println("Total amount: ");
    //Serial.println (totalAmount);
  }
  sei();
}

void animation1() {
  while (1) {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis >= interval) {
      previousMillis = currentMillis;
      //see what segment to display
      switch (currentPos) {
        case 0:
          sevseg.setSegmentsDigit(0, 0b00000001);
          break;
        case 1:
          sevseg.setSegmentsDigit(1, 0b00000001);
          sevseg.setSegmentsDigit(0, 0b00000000);
          break;
        case 2:
          sevseg.setSegmentsDigit(2, 0b00000001);
          sevseg.setSegmentsDigit(1, 0b00000000);
          break;
        case 3:
          sevseg.setSegmentsDigit(3, 0b00000001);
          sevseg.setSegmentsDigit(2, 0b00000000);
          break;
        case 4:
          sevseg.setSegmentsDigit(3, 0b00000010);
          break;
        case 5:
          sevseg.setSegmentsDigit(3, 0b00000100);
          break;
        case 6:
          sevseg.setSegmentsDigit(3, 0b00001000);
          break;
        case 7:
          sevseg.setSegmentsDigit(2, 0b00001000);
          sevseg.setSegmentsDigit(3, 0b00000000);
          break;
        case 8:
          sevseg.setSegmentsDigit(1, 0b00001000);
          sevseg.setSegmentsDigit(2, 0b00000000);
          break;
        case 9:
          sevseg.setSegmentsDigit(0, 0b00001000);
          sevseg.setSegmentsDigit(1, 0b00000000);
          break;
        case 10:
          sevseg.setSegmentsDigit(0, 0b00010000);
          break;
        case 11:
          sevseg.setSegmentsDigit(0, 0b00100000); /* 0b0GFEDCBA*/ /* Vertical bellow */
          break;
      }
      /* 0b0GFEDCBA*/
      currentPos += 1;
      if (currentPos > 12) {
        break;
      }
    }
    sevseg.refreshDisplay();
  }
}

void animation2() {
  while (1) {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis >= interval) {
      previousMillis = currentMillis;
      //see what segment to display
      switch (currentPos) {
        case 0:
          sevseg.setSegmentsDigit(0, 0b00110000);
          break;
        case 1:
          sevseg.setSegmentsDigit(0, 0b00000110);
          break;
        case 2:
          sevseg.setSegmentsDigit(1, 0b00110000);
          sevseg.setSegmentsDigit(0, 0b00000000);
          break;
        case 3:
          sevseg.setSegmentsDigit(1, 0b00000110);
          break;
        case 4:
          sevseg.setSegmentsDigit(2, 0b00110000);
          sevseg.setSegmentsDigit(1, 0b00000000);
          break;
        case 5:
          sevseg.setSegmentsDigit(2, 0b00000110);
          break;
        case 6:
          sevseg.setSegmentsDigit(3, 0b00110000);
          sevseg.setSegmentsDigit(2, 0b00000000);
          break;
        case 7:
          sevseg.setSegmentsDigit(3, 0b00000110);
          break;
        case 8:
          sevseg.setSegmentsDigit(3, 0b00110000);
          break;
        case 9:
          sevseg.setSegmentsDigit(2, 0b00000110);
          sevseg.setSegmentsDigit(3, 0b00000000);
          break;
        case 10:
          sevseg.setSegmentsDigit(2, 0b00110000);
          break;
        case 11:
          sevseg.setSegmentsDigit(1, 0b00000110);
          sevseg.setSegmentsDigit(2, 0b00000000);
          break;
        case 12:
          sevseg.setSegmentsDigit(1, 0b00110000);
          break;
        case 13:
          sevseg.setSegmentsDigit(0, 0b00000110);
          sevseg.setSegmentsDigit(1, 0b00000000);
          break;
        case 14:
          sevseg.setSegmentsDigit(0, 0b000110000);
          break;
      }
      /* 0b0GFEDCBA*/
      currentPos += 1;
      if (currentPos > 15) {
        break;
      }
    }
    sevseg.refreshDisplay();
  }
}
